/*
 * $Id: dnd_filesel.c,v 1.4 2002/02/20 16:04:24 stephen Exp $
 *
 * dnd_filesel.c - arrange for GtkFileSelection widgets to perform drag &
 * drop file saving and loading.
 *
 * Adapted by Stephen Watson, <stephen@kerofin.demon.co.uk>, from code
 * written by Thomas Leonard, <tal197@users.sourceforge.net>.
 */

/*
 * We assume we have a reasonable libc and the GTK+ 1.2.10 libraries available.
 * We do not use any other libraries, no matter how useful they would be...
 *
 * Use dnd_setup_load_sel(filesel, udata) to enable drag & drop loading for the
 * file selector.  Use dnd_setup_save_sel(filesel, udata) to enable drag & drop
 * saving for the file selector.
 *
 * They work by processing the DnD protocol until the filename has been
 * determined, then using gtk_file_selection_set_filename to store it and
 * triggering the "clicked" event for the OK button.  In each case the udata
 * parameter should be the same as that for the "clicked" signal handler for
 * the OK button.
 *
 * See test_dnd.c for an example.
 */

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <stdarg.h>

#include <sys/stat.h>

#include <gtk/gtk.h>

#include "dnd_filesel.h"

/* XPM of icon to use for dragging.  This is Choices/MIME-icons/text.xpm from
   rox-base
*/
/* XPM */
static char * unknown_xpm[] = {
"29 34 84 1",
" 	c None",
".	c #121212",
"+	c #000000",
"@	c #050505",
"#	c #151515",
"$	c #030303",
"%	c #FCFCFC",
"&	c #F9F9F9",
"*	c #F8F8F8",
"=	c #FAFAFA",
"-	c #FBFBFB",
";	c #F7F7F7",
">	c #FEFEFE",
",	c #F4F4F4",
"'	c #D5D5D5",
")	c #0B0B0B",
"!	c #060606",
"~	c #F5F5F5",
"{	c #F6F6F6",
"]	c #F0F0F0",
"^	c #F3F3F3",
"/	c #E5E5E5",
"(	c #C6C6C6",
"_	c #F2F2F2",
":	c #F1F1F1",
"<	c #EBEBEB",
"[	c #EEEEEE",
"}	c #EDEDED",
"|	c #E7E7E7",
"1	c #D3D3D3",
"2	c #B4B4B4",
"3	c #070707",
"4	c #EFEFEF",
"5	c #EAEAEA",
"6	c #ECECEC",
"7	c #E6E6E6",
"8	c #CACACA",
"9	c #ABABAB",
"0	c #010101",
"a	c #E9E9E9",
"b	c #C8C8C8",
"c	c #A9A9A9",
"d	c #090909",
"e	c #CDCDCD",
"f	c #080808",
"g	c #D0D0D0",
"h	c #FDFDFD",
"i	c #FFFFFF",
"j	c #CECECE",
"k	c #ACACAC",
"l	c #AAAAAA",
"m	c #040404",
"n	c #E8E8E8",
"o	c #CCCCCC",
"p	c #CBCBCB",
"q	c #E4E4E4",
"r	c #A8A8A8",
"s	c #E2E2E2",
"t	c #C7C7C7",
"u	c #A6A6A6",
"v	c #E1E1E1",
"w	c #E3E3E3",
"x	c #C5C5C5",
"y	c #A4A4A4",
"z	c #0C0C0C",
"A	c #A5A5A5",
"B	c #020202",
"C	c #E0E0E0",
"D	c #DFDFDF",
"E	c #C4C4C4",
"F	c #DDDDDD",
"G	c #D9D9D9",
"H	c #DADADA",
"I	c #C2C2C2",
"J	c #A7A7A7",
"K	c #C9C9C9",
"L	c #C0C0C0",
"M	c #B7B7B7",
"N	c #DBDBDB",
"O	c #A1A1A1",
"P	c #A2A2A2",
"Q	c #A3A3A3",
"R	c #A0A0A0",
"S	c #9D9D9D",
".+@#$@@@@@@@@@@@@@@@@@@@@@@@@",
"@%&*=&&===&&*----====&;>=&,')",
"!%*~{;;***;;{*;;;{{{~~]~^]/($",
"!-{__,~~{{~~,,^^__::::<[}|12+",
"3-{::^,~~~~~,^^_::]]4:5<67890",
"!%;^^,~{;;;;{~~,,^_:::66}abc3",
"!%*~~{;;*&&&&**;{~,^^4[66<8cd",
"!%*{;;;*&====&&*;~,^^[_}5}ecf",
"!-*{;;;*&=--=&*;{~^^_4~454gc!",
"@%&{;*&=%h>>i&&*{~,^_]:466jk!",
"@%&{;*&-%h>>i=&*;~,^^]:466jk!",
"!%*{{&=-%h>>>==&;{~,^]:466j9!",
"!%*~{&&-%hhhh==&*{~,^4:[<<j9!",
"!%*~~*&=-%%%%==&;{~,^4][<<e9@",
"!%*,~;*&=====&&*{~,^_4][<<el@",
"!%;,~{{;*******;~,^_:[]}55el@",
"!%;,,~{{;;;;;;;{~^_:][]}55el@",
"m={^^,~~{{{{{{~,^_:]4}[<naol@",
"@={_^,,~~{{~~~~,^:]44}[<nnpl@",
"@=~::_^,,~,,,,^^_]44[6}5|npc@",
"@=~]]::_^^^____:]4[[}<6a||pc@",
"!=,4[4]]::::]]]44[}}6a<n7|8cm",
"!=^}6}[44]444[[[}}666n5n/78cm",
"!=^6<6}}[[[[[}}}66<<<na|/78cm",
"!=^6566}}[[}}6666<<<<|a|q78rm",
"f~;<<66666}}}555aannnn/sq7tum",
"f{;5aaaaa5555aann||77n/vvwxy$",
"z*;n||||nnnnnnnn||77/7/svsxAm",
"z{,qw7777||||||77//qqswsvvxAB",
"d^_q/777||||n777//qqqCqwCDEy+",
"),:swsssswwwwssvvCCDDFvDGHIJ@",
"d<CptbbbbbKKKKKbbtt((EbxLxMAd",
"0NtcOuuuuuJJJrrJJuuAAPuQR9uS!",
"#$d+fmmmmmmmmmmmmmmmm@@@@@@@@"};

static gboolean done_init=FALSE;   /* Made our atoms? */
static gboolean hide_lists=FALSE;  /* Do we hide unwanted parts? */

/* Data we hold on the transfer */
struct client_data {
  guint flags;
  dnd_handle_uri uris;
  gpointer udata;       /* User data */
  gchar *uri;
};

typedef struct client_data ClientData;

enum {
	TARGET_RAW,
	TARGET_URI_LIST,
	TARGET_XDS,
	TARGET_STRING,
};

#define MAXURILEN 4096

static void init(void);
static void dnd_error(const char *fmt, ...);
static gboolean drag_drop(GtkWidget *widget, GdkDragContext *context,
			  gint x, gint y, guint time, gpointer data);
static void drag_data_received(GtkWidget *widget, GdkDragContext *context,
			       gint x, gint y,
			       GtkSelectionData *selection_data,
			       guint info, guint32 time, gpointer user_data);
static int drop_for_filesel(GtkWidget* widget, const char *uri, void *udata);
static void do_hide(GtkWidget *);
static void make_icon_pixmap(GtkWidget *window, const char *mime_type);
static void press_on_icon(GtkWidget *ebox, GdkEventButton *ev,
			  GtkFileSelection *fs);
static void drag_end(GtkWidget *widget, GdkDragContext *context,
		     GtkFileSelection *fs);
static void drag_data_get(GtkWidget *widget, GdkDragContext *context,
			  GtkSelectionData *data, guint info, guint time,
			  GtkFileSelection *fs);
static gchar *choices_find_path_load(const char *leaf, const char *dir);
static gboolean exists(const char *path);
static gchar *find_xpm_for_mime_type(const char *mime_type);
static void cleanup(void);

/* Atoms */
static GdkAtom XdndDirectSave0;
static GdkAtom xa_text_plain;
static GdkAtom text_uri_list;
static GdkAtom application_octet_stream;
static GdkAtom xa_string;

enum target_ids {
  MY_XDS_TARGET=0x6662112,
  MY_XDND_TARGET
};

/* Image of icon used to drag */
static GdkPixmap *pixmap=NULL;
static GdkBitmap *mask;

/* Save targets we support */
static GtkTargetList *targets=NULL;

/* List of temporary files we have created and should remove on exit */
static GSList *tmp_fnames=NULL;

/* Macro to make sure we have initialised */
#define check_init() do { if(!done_init) init();} while(0)

/*
 * Public functions
 */

/* Set whether or not to hide unneeded bits of the filesel */
void dnd_hide_file_lists(gboolean hide)
{
  hide_lists=hide;
}

/* This is where we act as the destination.  A generic set up */
void dnd_register_dest(GtkWidget *widget, guint flags,
		       dnd_handle_uri uris,
		       gpointer udata)
{
  /* Acceptable methods of receiving */
  static GtkTargetEntry target_table[]={
    {"text/uri-list", 0, TARGET_URI_LIST},
    {"XdndDirectSave0", 0, TARGET_XDS},
  };
  static const int ntarget=sizeof(target_table)/sizeof(target_table[0]);
  
  ClientData *rdata;
  guint dest_flags=GDK_ACTION_COPY|GDK_ACTION_PRIVATE;

  check_init();

  if(flags & DND_ALLOW_MOVE)
    dest_flags|=GDK_ACTION_MOVE;

  gtk_drag_dest_set(widget,
		      GTK_DEST_DEFAULT_MOTION|GTK_DEST_DEFAULT_HIGHLIGHT, 
		      target_table, ntarget, dest_flags);

  rdata=g_new(ClientData, 1);

  rdata->flags=flags;
  rdata->uris=uris;
  rdata->udata=udata;
  rdata->uri=NULL;

  gtk_signal_connect(GTK_OBJECT(widget), "drag_drop",
		     GTK_SIGNAL_FUNC(drag_drop), rdata);
  gtk_signal_connect(GTK_OBJECT(widget), "drag_data_received",
		     GTK_SIGNAL_FUNC(drag_data_received), rdata);
}

/* Use dnd_register_dest on the given fs.  drop_for_filesel does the clever
   stuff */
void dnd_setup_load_sel(GtkWidget *fs, gpointer udata)
{
  dnd_register_dest(fs, DND_DEFAULT, drop_for_filesel, udata);
  do_hide(fs);
}

/* Set up a draggable icon for saving, using XDS */
void dnd_setup_save_sel(GtkWidget *fs, gpointer udata)
{
  GtkWidget *pmap;
  GtkWidget *box;
  GtkWidget *vbox;
  
  check_init();

  g_return_if_fail(fs!=0);
  g_return_if_fail(GTK_IS_FILE_SELECTION(fs));

  /* Make the pixmap (if needed) from the default XPM */
  if(!pixmap) {
    gtk_widget_realize(fs);
    make_icon_pixmap(fs, NULL);
  }
  
  vbox=GTK_FILE_SELECTION(fs)->action_area;

  box=gtk_event_box_new();
  gtk_box_pack_start(GTK_BOX(vbox), box, FALSE, FALSE, 2);
  gtk_widget_show(box);
  gtk_widget_add_events(box, GDK_BUTTON_PRESS_MASK);
  gtk_signal_connect(GTK_OBJECT(box), "button_press_event",
		     GTK_SIGNAL_FUNC(press_on_icon), fs);
  /*gtk_signal_connect(GTK_OBJECT(fs), "drag_end",
		     GTK_SIGNAL_FUNC(drag_end), fs);*/
  gtk_signal_connect(GTK_OBJECT(fs), "drag_data_get",
		     GTK_SIGNAL_FUNC(drag_data_get), fs);

  pmap=gtk_pixmap_new(pixmap, mask);
  gtk_container_add(GTK_CONTAINER(box), pmap);
  gtk_widget_show(pmap);

  gtk_object_set_data(GTK_OBJECT(fs), "dnd_icon", pmap);
  gtk_object_set_data(GTK_OBJECT(fs), "dnd_user_data", udata);

  do_hide(fs);
}

/* Set the icon to use for this filesel, based on mime type */
void dnd_set_mime_type(GtkWidget *widget, const char *type)
{
  GtkFileSelection *fs;
  GtkWidget *pmap, *box;
  
  g_return_if_fail(GTK_IS_FILE_SELECTION(widget));
  if(!type)
    return;

  /* Get the old icon.  If one wasn't set, return without doing anything */
  fs=GTK_FILE_SELECTION(widget);
  pmap=gtk_object_get_data(GTK_OBJECT(fs), "dnd_icon");
  if(!pmap)
    return;
  g_return_if_fail(GTK_IS_PIXMAP(pmap));

  /* Make the pixmap of the new icon */
  if(!GTK_WIDGET_REALIZED(widget))
    gtk_widget_realize(widget);
  make_icon_pixmap(widget, type);

  /* Remove the old icon, put the new one in its place */
  box=pmap->parent;
  g_return_if_fail(GTK_IS_CONTAINER(box));
  gtk_container_remove(GTK_CONTAINER(box), pmap);
  /* gtk_widget_unref(pmap); */
  pmap=gtk_pixmap_new(pixmap, mask);
  gtk_container_add(GTK_CONTAINER(box), pmap);
  gtk_widget_show(pmap);
  gtk_object_set_data(GTK_OBJECT(fs), "dnd_icon", pmap);
}

/*
 * Private functions
 */

/* This is the error reporting routine, customize it for the program
   you are modifing */
static void dnd_error(const char *fmt, ...)
{
  gchar *message;
  va_list args;

  va_start(args, fmt);
  message=g_strdup_vprintf(fmt, args);
  va_end(args);

  /* Deal with the message how you see fit.
     By default we use g_warning and beep.  
  */
  g_warning(message);
  gdk_beep();

  g_free(message);
}

/* Initialise our atoms */
static void init(void)
{
  XdndDirectSave0=gdk_atom_intern("XdndDirectSave0", FALSE);
  xa_text_plain=gdk_atom_intern("text/plain", FALSE);
  text_uri_list=gdk_atom_intern("text/uri-list", FALSE);
  application_octet_stream=gdk_atom_intern("application/octet-stream",
					   FALSE);
  xa_string=gdk_atom_intern ("STRING", FALSE);

  done_init=TRUE;
}

static void do_hide(GtkWidget *widget)
{
  GtkFileSelection *fs;

  g_return_if_fail(GTK_IS_FILE_SELECTION(widget));

  fs=GTK_FILE_SELECTION(widget);
  
  if(hide_lists) {
    gtk_file_selection_hide_fileop_buttons(fs);

    /* Need hide the scrolling window the lists are in, not just the
       list itself */
    gtk_widget_hide(fs->dir_list->parent);
    gtk_widget_hide(fs->file_list->parent);
    
    gtk_widget_hide(fs->history_pulldown);
    
  } else {
    gtk_widget_show(fs->dir_list->parent);
    gtk_widget_show(fs->file_list->parent);
    
    gtk_widget_show(fs->history_pulldown);
  }
}

/* Is the sender willing to supply this target type? */
static gboolean provides(GdkDragContext *context, GdkAtom target)
{
  GList *targets = context->targets;

  while (targets && ((GdkAtom) targets->data != target))
    targets = targets->next;
  
  return targets != NULL;
}

static char *get_xds_prop(GdkDragContext *context)
{
  guchar *prop_text;
  gint length;

  if(gdk_property_get(context->source_window,
		       XdndDirectSave0,
		       xa_text_plain,
		       0, MAXURILEN,
		       FALSE,
		       NULL, NULL,
		       &length, &prop_text) && prop_text) {
    /* Terminate the string */
    prop_text = g_realloc(prop_text, length + 1);
    prop_text[length] = '\0';
    return prop_text;
  }

  return NULL;
}

/* Set the XdndDirectSave0 property on the source window for this context */
static void set_xds_prop(GdkDragContext *context, char *text)
{
  gdk_property_change(context->source_window,
		      XdndDirectSave0,
		      xa_text_plain, 8,
		      GDK_PROP_MODE_REPLACE,
		      text,
		      strlen(text));
}

/* Convert a list of URIs into a list of strings.
 * Lines beginning with # are skipped.
 * The text block passed in is zero terminated (after the final CRLF)
 */
static GSList *uri_list_to_gslist(char *uri_list)
{
  GSList   *list = NULL;

  while (*uri_list) {
    char	*linebreak;
    char	*uri;
    int	length;

    linebreak = strchr(uri_list, 13);

    if (!linebreak || linebreak[1] != 10) {
      return list;
    }
    
    length = linebreak - uri_list;

    if (length && uri_list[0] != '#') {
	uri = g_malloc(sizeof(char) * (length + 1));
	strncpy(uri, uri_list, length);
	uri[length] = 0;
	list = g_slist_append(list, uri);
      }

    uri_list = linebreak + 2;
  }

  return list;
}

/* User has tried to drop some data on us. Decide what format we would
 * like the data in.
 */
static gboolean drag_drop(GtkWidget *widget, GdkDragContext *context,
			  gint x, gint y, guint time, gpointer data)
{
  char *leafname=NULL;
  char *path=NULL;
  GdkAtom target = GDK_NONE;
  ClientData *rdata=(ClientData *) data;

  if(provides(context, XdndDirectSave0)) {
    leafname = get_xds_prop(context);
    if (leafname) {
      gchar *uri;
      char host[1025];

      gethostname(host, sizeof(host));
      uri=g_strconcat("file://", host, "/tmp/", leafname, NULL);
      set_xds_prop(context, uri);
      if(rdata->uri)
	g_free(rdata->uri);
      rdata->uri=uri;

      /* Clean up the temp file when we exit */
      if(!tmp_fnames)
	atexit(cleanup);
      tmp_fnames=g_slist_prepend(tmp_fnames,
				 g_strconcat("/tmp/", leafname, NULL));
      
      target = XdndDirectSave0;
      g_dataset_set_data_full(context, "leafname", leafname, g_free);
      /*printf("drag drop %s %s\n", uri, leafname);*/
    }
  } else if(provides(context, text_uri_list))
    target = text_uri_list;

  gtk_drag_get_data(widget, context, target, time);

  return TRUE;
}

/* We've got a list of URIs from somewhere (probably a filer window).
 * If the files are on the local machine then use the name,
 */
static void got_uri_list(GtkWidget *widget, GdkDragContext *context,
			 GtkSelectionData *selection_data,
			 guint32 time, ClientData *rdata)
{
  GSList *uri_list;
  char *error = NULL;
  gboolean ok;

  uri_list = uri_list_to_gslist(selection_data->data);

  if (!uri_list) {
    ok=FALSE;
    error = "No URIs in the text/uri-list (nothing to do!)";
  } else if(rdata && rdata->uris) {
    GSList *file;

    ok=TRUE;
    for(file=uri_list; file; file=g_slist_next(file)) {
      ok=ok && rdata->uris(widget, file->data, rdata->udata);
    }
  }

  if (error) {
    dnd_error("Can't process URI list: %s", error);
  }
  gtk_drag_finish(context, ok, FALSE, time);
  
  g_slist_foreach(uri_list, (GFunc) g_free, NULL);
  g_slist_free(uri_list);
}

static char *dnd_get_local(const char *uri)
{
  char *host;
  char *end;
  char hostn[256];
  const char *orig=uri;
  
  if(strncmp(uri, "file:", 5)==0)
    uri+=5;
  
  if(uri[0]=='/') {
    if(uri[1]!='/')
      return g_strdup(uri);
    if(uri[2]=='/')
      return g_strdup(uri+2);

    host=(char *) uri+2;
    end=strchr(host, '/');
    host=g_strndup(host, end-host);

    if(strcmp(host, "localhost")==0) {
      g_free(host);
      return g_strdup(end);
    }

    if(gethostname(hostn, sizeof(hostn))==0 && strcmp(host, hostn)==0) {
      g_free(host);
      return g_strdup(end);
    }

    g_free(host);
  }

  return NULL;
}

static void got_xds(GtkWidget 		*widget,
			 GdkDragContext 	*context,
			 GtkSelectionData 	*selection_data,
			 guint32             	time,
			 ClientData *rdata)
{
  char response=*selection_data->data;
  gboolean saved=FALSE;

  /*printf("got_xds response=%c %d\n", response, response);*/
  switch(response) {
  case 'F':
    /* Sender couldn't save there - ask for another
     * type if possible.
     */
    if (provides(context, text_uri_list)) {
      gtk_drag_get_data(widget, context, text_uri_list, time);
      return;
    } else {
      dnd_error("Drag & drop error: sender can't provide data");
    }
    break;

  case 'S':
    saved=TRUE;
    
    if(rdata->uris) {
      if(!rdata->uris(widget, rdata->uri, rdata->udata))
	saved=FALSE;
    } 
    break;
    
  case 'E':
    break;

  default:
    dnd_error("XDS protocol error, '%c' should be S, F or E", response);
    break;
  }

  if(!saved)
    gtk_drag_finish(context, FALSE, FALSE, time);
  else
    gtk_drag_finish(context, TRUE, FALSE, time);
}

/* Called when some data arrives from the remote app (which we asked for
 * in drag_drop).
 */
static void drag_data_received(GtkWidget      	*widget,
			       GdkDragContext  	*context,
			       gint            	x,
			       gint            	y,
			       GtkSelectionData *selection_data,
			       guint            info,
			       guint32          time,
			       gpointer		user_data)
{
  if (!selection_data->data) {
    /* Timeout? */
    gtk_drag_finish(context, FALSE, FALSE, time);	/* Failure */
    return;
  }

  switch(info) {
  case TARGET_XDS:
    got_xds(widget, context, selection_data, time, (ClientData *) user_data);
    break;
  case TARGET_URI_LIST:
    got_uri_list(widget, context, selection_data, time,
		 (ClientData *) user_data);
    break;
  default:
    dnd_error("drag_data_received: unknown target %u", info);
    gtk_drag_finish(context, FALSE, FALSE, time);
    break;
  }
}

static int drop_for_filesel(GtkWidget* widget,
			    const char *uri,
			    void *udata)
{
  char *local;

  local=dnd_get_local(uri);
  /*printf("%s %s\n", uri, local? local: "(NULL)");*/
  if(local) {
    gboolean state=FALSE;

    if(GTK_IS_FILE_SELECTION(widget)) {
      gtk_file_selection_set_filename(GTK_FILE_SELECTION (widget), local);
      gtk_signal_emit_by_name(GTK_OBJECT(GTK_FILE_SELECTION(widget)->ok_button),
			      "clicked", udata);
      state=TRUE;
    }

    g_free(local);

    return state;
  } else {
    dnd_error("Cannot handle non-local URI's (%s)", uri);
  }

  return FALSE;
}

/* This is where we act as the source */
static void make_icon_pixmap(GtkWidget *window, const char *mime_type)
{
  GtkStyle *style;
  gchar *xpm_file;
  
  style=gtk_widget_get_style(window);

  if(pixmap) {
    gdk_pixmap_unref(pixmap);
    gdk_bitmap_unref(mask);
    pixmap=NULL;
  }

  xpm_file=find_xpm_for_mime_type(mime_type);
  if(xpm_file) {
    pixmap=gdk_pixmap_create_from_xpm(GTK_WIDGET(window)->window,  &mask,
				      &style->bg[GTK_STATE_NORMAL],
				      xpm_file);
    g_free(xpm_file);
  }

  if(!pixmap) {
    pixmap=gdk_pixmap_create_from_xpm_d(GTK_WIDGET(window)->window,  &mask,
					&style->bg[GTK_STATE_NORMAL],
					(gchar **) unknown_xpm);
  }
}

static void press_on_icon(GtkWidget *ebox, GdkEventButton *ev,
			  GtkFileSelection *fs)
{
  gchar *path, *leaf;
  GdkDragContext *context;
  GtkWidget *icon;
  GtkTargetEntry targent[] = {
    {"XdndDirectSave0", 0, MY_XDS_TARGET},
    /*{"text/uri-list", 0, MY_XDND_TARGET}*/
  };

  if(!targets) {
    targets=gtk_target_list_new(targent, sizeof(targent)/sizeof(*targent));
  }
  
  path=gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs));
  leaf=strrchr(path, '/');
  if(leaf)
    leaf++;
  else
    leaf=path;
  leaf=g_strdup(leaf);

  context=gtk_drag_begin(GTK_WIDGET(fs), targets, GDK_ACTION_COPY,
			 ev->button, (GdkEvent *) ev);
  icon=GTK_WIDGET(gtk_object_get_data(GTK_OBJECT(fs), "dnd_icon"));
  gtk_drag_set_icon_pixmap(context, gtk_widget_get_colormap(icon),
			   pixmap, mask, ev->x, ev->y);

  gdk_property_change(context->source_window, XdndDirectSave0,
		      xa_text_plain, 8, GDK_PROP_MODE_REPLACE,
		      leaf, strlen(leaf));
}

static void drag_end(GtkWidget *widget, GdkDragContext *context,
		     GtkFileSelection *fs)
{
  GList *targets;
  
  for(targets=context->targets; targets; targets=g_list_next(targets)) {
    GdkAtom atom=(GdkAtom) targets->data;
    gchar *name=gdk_atom_name(atom);

    /* Do something? */
    
    g_free(name);
  }
}

static void drag_data_get(GtkWidget *widget, GdkDragContext *context,
			  GtkSelectionData *data, guint info, guint time,
			  GtkFileSelection *fs)
{
  GList *targets;
  gchar *name;
  GdkAtom prop;
  gint format, length;
  guchar *cdata=NULL;
  char xds_reply='E';
  gpointer udata;

  udata=gtk_object_get_data(GTK_OBJECT(fs), "dnd_user_data");
			    
  if(gdk_property_get(context->source_window, XdndDirectSave0, xa_text_plain,
		      0, 1024, FALSE, &prop, &format, &length, &cdata)) {
    if(cdata && format==8)
      cdata[length]=0;
    name=gdk_atom_name(prop);
    /*printf("drag_data_get name=%s cdata=%p %s\n", name, cdata,
	   cdata? (char *) cdata: "");*/

    if(cdata && format==8 && strncmp(cdata, "file://", 7)==0) {
      char *path;
      char *sep;

      path=dnd_get_local(cdata);

      if(!path) {
	xds_reply='F';
      } else {
	gtk_file_selection_set_filename(GTK_FILE_SELECTION (fs), path);
	gtk_signal_emit_by_name(GTK_OBJECT(GTK_FILE_SELECTION(fs)->ok_button),
				"clicked", udata);
	xds_reply='S';
	g_free(path);
      }
    }
    g_free(name);
    g_free(cdata);
  }
  
  gtk_selection_data_set(data, xa_string, 8, &xds_reply, 1);
}

static gchar *find_xpm_for_mime_type(const char *mime_type)
{
  gchar *fname=NULL;
  gchar **tok, *leaf=NULL;

  if(!mime_type)
    return NULL;

  tok=g_strsplit(mime_type, "/", 3);
  if(tok[2] && tok[2][0]) {
    g_strfreev(tok);
    return NULL;
  }

  if(tok[1] && tok[1][0]) {
    leaf=g_strconcat(tok[0], "_", tok[1], ".xpm", NULL);
    fname=choices_find_path_load(leaf, "MIME-icons");
    g_free(leaf);
  }
  if(!fname && tok[0] && tok[0][0]) {
    leaf=g_strconcat(tok[0], ".xpm", NULL);
    fname=choices_find_path_load(leaf, "MIME-icons");
    g_free(leaf);
  }

  g_strfreev(tok);
  return fname;
}

static void cleanup(void)
{
  while(tmp_fnames) {
    unlink((const char *) tmp_fnames->data);
    tmp_fnames=g_slist_next(tmp_fnames);
  }
}

/* Short choices implementation */
static gchar **dir_list = NULL;

/* Reads in CHOICESPATH and constructs the directory list table.
 * You must call this before using any other choices_* functions.
 *
 * If CHOICESPATH does not exist then a suitable default is used.
 */
static void choices_init(void)
{
	char	*choices;

	if(dir_list)
	  return;

	choices = getenv("CHOICESPATH");
	
	if (choices)
	{
		while (*choices == ':')
			choices++;

		if (*choices == '\0')
		{
			dir_list = g_new(char *, 1);
			dir_list[0] = NULL;
		}
		else
			dir_list = g_strsplit(choices, ":", 0);
	}
	else
	{
		dir_list = g_new(gchar *, 4);
		dir_list[0] = g_strconcat(getenv("HOME"), "/Choices", NULL);
		dir_list[1] = g_strdup("/usr/local/share/Choices");
		dir_list[2] = g_strdup("/usr/share/Choices");
		dir_list[3] = NULL;
	}

}

/* Get the pathname of a choices file to load. Eg:
 *
 * choices_find_path_load("menus", "ROX-Filer")
 *		 		-> "/usr/local/share/Choices/ROX-Filer/menus".
 *
 * The return values may be NULL - use built-in defaults - otherwise
 * g_free() the result.
 */
gchar *choices_find_path_load(const char *leaf, const char *dir)
{
	gchar		**cdir;

	if(!dir_list)
	  choices_init();

	cdir = dir_list;

	while (*cdir)
	{
		gchar	*path;

		path = g_strconcat(*cdir, "/", dir, "/", leaf, NULL);

		if (exists(path))
			return path;

		g_free(path);

		cdir++;
	}

	return NULL;
}

/* Returns TRUE if the object exists, FALSE if it doesn't */
static gboolean exists(const char *path)
{
	struct stat info;

	return stat(path, &info) == 0;
}


/*
 * $Log: dnd_filesel.c,v $
 * Revision 1.4  2002/02/20 16:04:24  stephen
 * Allow for use by C++
 * Tidy up temp files on exit.
 *
 * Revision 1.3  2002/02/01 12:53:50  stephen
 * Incorporated some suggestions from rox-devel.
 * Call this release 0.1.1
 *
 */

