/*
 * $Id: dnd_filesel.h,v 1.4 2002/02/20 16:04:24 stephen Exp $
 *
 * Adapted by Stephen Watson, <stephen@kerofin.demon.co.uk>, from code
 * written by Thomas Leonard, <tal197@users.sourceforge.net>.
 */

#ifndef _dnd_filesel_h
#define _dnd_filesel_h

#ifdef __cplusplus
extern "C" {
#endif

/*
 * We assume we have a reasonable libc and the GTK+ 1.2.10 libraries available.
 * We do not use any other libraries, no matter how useful they would be...
 *
 * Use dnd_setup_load_sel(filesel, udata) to enable drag & drop loading for the
 * file selector.  Use dnd_setup_save_sel(filesel, udata) to enable drag & drop
 * saving for the file selector.
 *
 * They work by processing the DnD protocol until the filename has been
 * determined, then using gtk_file_selection_set_filename to store it and
 * triggering the "clicked" event for the OK button.  In each case the udata
 * parameter should be the same as that for the "clicked" signal handler for
 * the OK button.
 *
 * See test_dnd.c for an example.
 */

/* Flags for dnd_register_dest */
#define DND_DEFAULT    0u   /* Standard options */
#define DND_ALLOW_MOVE 1u   /* The file can be moved */

/*
 * Function which responds to drop on a widget by processing a URI.
 * Return TRUE if handled successfully, FALSE otherwise.
 * May get called multiple times if multiple file drops
 */
typedef int (*dnd_handle_uri)(GtkWidget* widget,
			       const char *uri,
			       gpointer udata);

/* Register a widget to handle a drop */
extern void dnd_register_dest(GtkWidget* widget, unsigned int flags,
			      dnd_handle_uri, gpointer udata);

/* Set up a GtkFileSelection to handle drops.  udata is the value to pass as
 the second parameter to the OK button's clicked signal */
extern void dnd_setup_load_sel(GtkWidget *fs, gpointer udata);
extern void dnd_setup_save_sel(GtkWidget *fs, gpointer udata);

/* If hide is true then the widget's file lists are hidden when
   dnd_setup_load_sel or dnd_setup_save_sel is called */
extern void dnd_hide_file_lists(gboolean hide);

/* Set the icon to use for saving by specifing the mime type.  Call it
   after dnd_setup_save_sel, but before showing the widget */
extern void dnd_set_mime_type(GtkWidget *fs, const char *type);

#ifdef __cplusplus
}
#endif

#endif

/*
 * $Log: dnd_filesel.h,v $
 * Revision 1.4  2002/02/20 16:04:24  stephen
 * Allow for use by C++
 * Tidy up temp files on exit.
 *
 * Revision 1.3  2002/02/01 12:53:50  stephen
 * Incorporated some suggestions from rox-devel.
 * Call this release 0.1.1
 *
 */
